globals [
  fast average slow avg-speed       ;; current counts
  avg-speed-top avg-speed-bottom    ;; current speed averages
  avg-energy-top avg-energy-bottom  ;; current energy averages
  clock vsplit vclock               ;; clock variables
  box-edge                          ;; edge length of box
]

turtles-own [
  speed mass energy new-speed            ;; turtle info
  v1t v1l tmp-turtle                     ;; collision info (turtle 1)
  heading2 mass2 speed2 v2t v2l turtle2  ;; collision info (turtle 2)
  theta                                  ;; collision info (both turtles)
]

to setup
  ca
  set box-edge 35    
  make-box
  set clock 0
  set vclock 0
  cct number [
    set new-speed initspeed
    set mass initmass
    random-position
    rt random 360
    set shape "circle"
    recolor
  ]
  update-variables
  setup-plots
  do-plotting
end

to update-variables
  locals [ top-turtles bottom-turtles ]

  ask turtles
    [ set speed new-speed
      set energy (0.5 * speed * speed * mass) ]

  set top-turtles    turtles with [ycor >= 0]
  set bottom-turtles turtles with [ycor <  0]
  if (not any top-turtles) or (not any bottom-turtles) [ stop ]
  set avg-speed         sum values-from turtles        [speed]  / count turtles
  set avg-speed-top     sum values-from top-turtles    [speed]  / count top-turtles
  set avg-speed-bottom  sum values-from bottom-turtles [speed]  / count bottom-turtles
  set avg-energy-top    sum values-from top-turtles    [energy] / count top-turtles
  set avg-energy-bottom sum values-from bottom-turtles [energy] / count bottom-turtles

  set average count turtles with [color = green]
  set slow    count turtles with [color = blue]
  set fast    count turtles with [color = red]
  set vsplit  round (1.2 * max values-from turtles [speed])
end

to go
  if (not any turtles with [ycor >= 0]) or
     (not any turtles with [ycor < 0])
    [ stop ]
  ask turtles [ bounce ]
  ask turtles [ move ]
  if demon?
  [ ask turtles [ maxwells-demon ] ]
  set vclock (vclock + 1)
  if (vclock = vsplit)
  [ set clock  (clock + 1)
    set vclock 0
    update-variables
    do-plotting
  ]
end

to bounce ;; turtle procedure
  locals [ new-px new-py ]
  ; if we're not about to hit a wall (yellow patch),
  ; we don't need to do any further checks
  if pcolor-of patch-at dx dy != yellow [ stop ]
  ; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  ; check: hitting left or right wall?
  if (abs new-px = box-edge)
    ; if so, reflect heading around x axis
    [ set heading (0 - heading) ]
  ; check: hitting top, bottom or middle wall?
  if (abs new-py = box-edge) or (new-py = 0)
    ; if so, reflect heading around y axis
    [ set heading (180 - heading) ]
end

to move  ;;turtle procedure
  jump (speed / vsplit)
  check-for-collision
end

;;this is the piece that's different from other GasLab models.  It 
;;creates a "valve" that moves molecules from one chamber to the other,
;; depending on their speed.
to maxwells-demon ;;patch procedure
  if ((pcolor = turquoise) and ((new-speed / threshold) > avg-speed-top))
   [set ycor -2]
  if ((pcolor = violet) and ((new-speed * threshold) < avg-speed-bottom))
   [set ycor 2]
end

to check-for-collision ;;turtle procedure
  if count other-turtles-here = 1
    [ set tmp-turtle random-one-of other-turtles-here
      if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
        [ collide ]
    ]
end

to collide  ;;turtle procedure
  get-turtle2-info
  calculate-velocity-components
  set-new-speed-and-headings
end

to get-turtle2-info  ;; Turtle procedure
  set turtle2 tmp-turtle
  set mass2 mass-of turtle2
  set speed2 new-speed-of turtle2
  set heading2 heading-of turtle2
end

to calculate-velocity-components
  locals [ vcm ]  ;; CM vel. along dir. theta
  set theta (random 360)
  set v1l (new-speed * sin (theta - heading))
  set v1t (new-speed * cos (theta - heading))
  set v2l (speed2 * sin (theta - heading2))
  set v2t (speed2 * cos (theta - heading2))
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2))
  set v1t (vcm + vcm - v1t)
  set v2t (vcm + vcm - v2t)
end

to set-new-speed-and-headings  ;;turtle procedure
  set new-speed sqrt ((v1t * v1t) + (v1l * v1l))
  set heading (theta - (atan v1l v1t)) 

  set new-speed-of turtle2 sqrt ((v2t * v2t) + (v2l * v2l))
  set heading-of turtle2 (theta - (atan v2l v2t))

  recolor
  ask turtle2 [ recolor ]
end

to recolor  ;; turtle procedure
  ifelse new-speed < (0.5 * initspeed)
    [ set color blue ]
    [ ifelse new-speed > (1.5 * initspeed)
        [ set color red ]
        [ set color green ] ]
end

to make-box
  ask patches
  [ if ((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
       ((abs pycor = box-edge) and (abs pxcor <= box-edge)) or
       ((pycor = 0) and (abs pxcor <= box-edge))
    [ set pcolor yellow ]
    ;; Maxwell's Demon
    if ((pxcor > -5) and (pxcor < 5) and (pycor = 1))
    [ set pcolor turquoise ]
    if ((pxcor > -5) and (pxcor < 5) and (pycor = -1))
    [ set pcolor violet ]
  ]
end

to random-position ;; turtle procedure
  setxy ((1 - box-edge) + random (2 * box-edge - 1))
        ((1 - box-edge) + random (2 * box-edge - 1))
  ; make sure none are created in the wall
  if pycor = 0
  [ random-position ]
end

;;; plotting procedures

to setup-plots
  set-current-plot "Average Speeds"
  set-plot-y-range 0 (1.5 * initspeed)
  set-current-plot "Molecule Count"
  set-plot-y-range (number / 2 - 100) (number / 2 + 100)
  set-current-plot "Average Energies"
  set-plot-y-range 0 (1.5 * initspeed * initspeed * initmass)
end

to do-plotting
  plot-speeds
  plot-molecules
  plot-energies  
end

to plot-speeds
  set-current-plot "Average Speeds"
  set-current-plot-pen "top"
  plot avg-speed-top
  set-current-plot-pen "bottom"
  plot avg-speed-bottom
end

to plot-molecules
  set-current-plot "Molecule Count"
  set-current-plot-pen "top"
  plot count turtles with [ycor >= 0]
  set-current-plot-pen "bottom"
  plot count turtles with [ycor < 0]
end

to plot-energies
  set-current-plot "Average Energies"
  set-current-plot-pen "top"
  plot avg-energy-top
  set-current-plot-pen "bottom"
  plot avg-energy-bottom
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo GasLab Maxwells Demon model.
; http://ccl.northwestern.edu/netlogo/models/GasLabMaxwellsDemon.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
275
10
609
365
40
40
4.0
0
10
0
0

CC-WINDOW
613
203
805
365
Command Center

BUTTON
9
41
64
74
setup
setup
NIL
1
T
OBSERVER

SLIDER
137
127
271
160
initspeed
initspeed
1.0
20.0
10.0
1.0
1
NIL

SLIDER
137
170
271
203
initmass
initmass
1.0
20.0
5.0
1.0
1
NIL

SWITCH
9
84
128
117
demon?
demon?
0
1
-1000

SLIDER
137
41
271
74
threshold
threshold
0.0
100.0
2.0
1.0
1
NIL

SLIDER
137
84
271
117
number
number
1
2000
500
1
1
NIL

MONITOR
77
213
164
262
avg-speed
avg-speed
3
1

MONITOR
9
272
118
321
avg-speed-top
avg-speed-top
3
1

MONITOR
9
213
68
262
clock
clock
3
1

MONITOR
126
272
253
321
avg-speed-bottom
avg-speed-bottom
3
1

BUTTON
73
41
128
74
go
go
T
1
T
OBSERVER

PLOT
7
377
238
551
Average Speeds
Time
Speed
0.0
20.0
0.0
15.0
true
false
PENS
"top" 1.0 0 -16711738 true
"bottom" 1.0 0 -8716033 true

PLOT
487
377
742
551
Molecule Count
Time
Molecules
0.0
20.0
150.0
350.0
true
true
PENS
"top" 1.0 0 -16711738 true
"bottom" 1.0 0 -8716033 true

PLOT
247
377
478
551
Average Energies
Time
Energy
0.0
20.0
0.0
750.0
true
false
PENS
"top" 1.0 0 -16711738 true
"bottom" 1.0 0 -8716033 true

@#$#@#$#@
WHAT IS IT?
-----------
This program illustrates a famous thought experiment which raised important issues about the nature of entropy.  In 1871, James Clerk Maxwell imagined a situation where a very small and nimble being could, by opening and closing a valve between two gas-filled chambers, allow only faster molecules to go through one way and only slower molecules to go through the other.  By doing this, the being (later called a demon) could raise the temperature of one chamber and lower the temperature of the other, without the expenditure of work.  This was a violation of the Second Law of Thermodynamics, and he asserted that this could not actually occur.  The implications of this puzzle have continued to be central to thermodynamics, entropy, and information theory up to the present time.  

The collision and bouncing rules for this model are the same as in the other GasLab models.  The setup is the same as "Two-Gas".  What is added is a "valve".  It transports fast molecules from the top to the bottom chamber when they arrive at the turquoise strip, and slow molecules from the bottom to the top chamber when they arrive at the violet strip.      


HOW TO USE IT
-------------
Set the total number of molecules (NUMBER), their initial speed (INITSPEED) and  mass (INITMASS), and the THRESHOLD sliders.  The threshold is how much faster or slower than the average speed a molecule must be going to be transferred to the other chamber by the "valve". Initialize the model by pressing SETUP, and press GO to run it. 

DEMON?:  If this is On, the valve is operating.  If it is Off, the valve does nothing. 

Monitors:
CLOCK:  number of ticks that have run.
AVG-SPEED:  average speed of all the molecules.
AVG-SPEED-TOP: average molecular speed in the upper chamber.
AVG-SPEED-BOTTOM: average molecular speed in the lower chamber.

Plots: 
MOLECULE COUNT: the number of molecules in the top (turquoise) and bottom (violet) chambers.
AVG ENERGIES: the average energy of the molecules in the upper and lower chambers.  This is calculated as the average of 1/2 mv^2.
AVG SPEEDS:  the average speeds of the molecules in the upper and the lower chambers.


THINGS TO NOTICE
----------------
Watch the "valve" carefully. Can you see fast (red) molecules jump downward and slow (blue) molecules jump upward when they run into the valve?

What happens to the average energies and speeds in the upper and lower chambers?  What about the molecule count?  Do the values settle down after a long time?  


THINGS TO TRY
-------------
Change the valve threshold.  Does it change the rate of evolution of the model or its eventual result?  


EXTENDING THE MODEL
-------------------
What happens if there are molecules of different masses?  (See Two Gas model.)

How would you calculate the pressure in each box?  (See Piston models.)  Do they remain equal?  

If such a valve were possible, could "free energy" be gotten from the two chambers at different temperatures?  

RELATED MODELS
---------------
Look at the other GasLab models.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo GasLab Maxwells Demon model. http://ccl.northwestern.edu/netlogo/models/GasLabMaxwellsDemon. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150
Polygon -7566196 true true 270 14
Polygon -7566196 true true 276 21

arrow
true
0
Polygon -7566196 true true 150 300 300 150 195 150 195 7 105 7 105 150 105 150 0 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 300 120 165 45 45 75 60 150 120 225 60 255 45 180 165

thin-arrow
true
0
Polygon -7566196 true true 150 300 300 150 180 150 180 4 120 4 120 150 120 150 0 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 46 130
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 1.3beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
