globals [
  ppatches  ;; like "patches", but excludes the top row and rightmost column
  lattice   ;; only those ppatches where both pxcor and pycor are even
]

to setup
  ca
  ask patches
    [ set pcolor white ]
  set ppatches patches with [pxcor < screen-edge-x and
                             pycor < screen-edge-y]
  set lattice ppatches with [pxcor mod 2 = 0 and pycor mod 2 = 0]
  ask ppatches
    [ if random 100.0 < density [ set pcolor black ] ]
end

to go
  no-display
  ask lattice [ do-rule  1 ]  ;; propagation
  ask lattice [ do-rule -1 ]  ;; collision
  display
end

to go-reverse  ;; applying rules to the lattice in reverse order reverses the system
  no-display
  ask lattice [ do-rule -1 ]  ;; collision
  ask lattice [ do-rule  1 ]  ;; propagation
  display
end

;; grid = 1 if even lattice, grid = -1 if odd lattice
to do-rule [grid]  
  locals [a b c d]
  set a ppatchat 0 0
  set b ppatchat (0 - grid) 0
  set c ppatchat 0 grid
  set d ppatchat (0 - grid) grid
  
  ifelse (pcolor-of a) != (pcolor-of b) and (pcolor-of c) != (pcolor-of d)
  [
    swap-pcolor a b
    swap-pcolor c d
  ]
  [
    swap-pcolor a d
    swap-pcolor b c
  ]
end

to swap-pcolor [x y]
  locals [temp]
  set temp pcolor-of x
  set (pcolor-of x) (pcolor-of y)
  set (pcolor-of y) temp
end

to draw-circle
  if mouse-down?
  [
    ask patch (round mouse-xcor) (round mouse-ycor)
    [
      ask ppatches in-radius radius [ set pcolor black ]
    ]
  ]
end

;;;
;;; what follows is support code for operating on an even-sized grid
;;;

to-report ppatchat [xoff yoff]  ;; patch procedure
  report patch-at adjust-x-offset xoff
                  adjust-y-offset yoff
end

to-report adjust-x-offset [xoff]  ;; patch procedure
  ;; if we landed in the rightmost column, go one patch farther
  ifelse pxcor-of patch-at xoff 0 = screen-edge-x
    [ report xoff * 2 ] ;; changes 1 to 2, -1 to -2
    [ report xoff ]
end

to-report adjust-y-offset [yoff]  ;; patch procedure
  ;; if we landed in the top row, go one patch farther
  ifelse pycor-of patch-at 0 yoff = screen-edge-y
    [ report yoff * 2 ] ;; changes 1 to 2, -1 to -2
    [ report yoff ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Lattice Gas Automaton model.
; http://ccl.northwestern.edu/netlogo/models/LatticeGasAutomaton.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
321
10
643
332
80
80
2.0
1
10
0
0

CC-WINDOW
8
214
277
319
Command Center

BUTTON
179
20
280
53
NIL
setup
NIL
1
T
OBSERVER

BUTTON
12
128
155
205
NIL
go
T
1
T
OBSERVER

SLIDER
7
20
179
53
density
density
0.0
100.0
17.0
1.0
1
%

BUTTON
179
60
280
93
NIL
draw-circle
T
1
T
OBSERVER

SLIDER
7
60
179
93
radius
radius
0
20
10
1
1
NIL

BUTTON
156
128
280
205
reverse
go-reverse
T
1
T
OBSERVER

@#$#@#$#@
WHAT IS IT?
-----------
This model demonstrates circular wave propogation that approximates the Navier-Stokes equation, a well established fluid dynamics equation discovered in 1823. In the model, space is divided in to cells which are occupied by the "fluid's" particles.  Each particle has the same mass and absolute velocity (each particle will move only a single cell at each time step.)  Space is broken up into neighborhoods of 2x2 cell squares. Collisions occur when multiple particles are in the same square and occur instantaniously, conserving both mass and momentum.

The model is implemented using a Margolus neighborhood of 2x2 cell squares in which the particles belong to two separate spacetime sublattices, propagation and collision, (sometimes refered to as "even" and "odd" lattices), which evolve independently of each other.


HOW IT WORKS
------------
The CA can be summarized with the following core rules, which are applied first to the "even" lattice of 2x2 squares and then applied to the "odd" lattice of 2x2 squares:

|   W W  ->  W W		B W  ->  W W		B W  ->  W B
|   W W      W W		W W      W B		W B      B W
|
|   B W  ->  W B		W B  ->  B B		B B  ->  B B
|   B W      W B		B B      B W		B B      B B

It applies these rules to an even lattice followed by an odd lattice, meaning that the top left patch of four applies one of the above rules, then the bottom right patch of four does the same. "Reverse" simply means switching the order of those two operations. No patch gets changed more then twice per tick. That means that each patch sees only one possible even rule and one possible odd rule each iteration.


HOW TO USE IT
-------------
SETUP - Sets up screen with a given percentage of particles
DENSITY - Percentage of particles in gas
DRAW-CIRCLE - Clicking any location on the screen creates a solid circle of particles
RADIUS - The radius of circles to be drawn at the given location
GO - Run the model
REVERSE - Run the model in reverse


THINGS TO NOTICE
----------------
Drawing a circle in the center of the screen and clicking GO creates a circular wave that travels through the lattice.  Once the wave reaches the edges of the screen, it wraps around the sides and causes the wave to collide with itself. You may stop the model by clicking GO again, and reverse the system by clicking REVERSE.  The wave will now implode and eventually return to its original starting state.

How does the density of the center of the wave vary over time? Why?

Why does the model appear to act the same when run forwards and backwards?


THINGS TO TRY
-------------
How does the density of the gas effect the propagation of the wave?  Try running the model with various DENSITY settings. Why do certain densities impede the propagation of waves?

What happens when you create multiple compressions in the gas and run the model?


EXTENDING THE MODEL
-------------------
Can you create obstacles that deflect the movement of particles?

The HPP model is very limited. The FHP (Frisch, Hasslacher and Pomeau) model was invented in the mid-eighties in order to improve the accuracy of the HPP model.  The underlying rules of the FHP model are similar to that of the HPP model except that the FHP model has a symmetry-group order of six.  The hexagonal lattice allows for more advanced modeling, such as hydrodynamical simulations.  Can you write a model that emulates a hexagonal lattice in NetLogo?


NETLOGO FEATURES
----------------
In order for the algorithm to operate correctly, the size of the lattice must be even in both dimensions.  However, the current version of NetLogo only supports odd sized grids.  Therefore, it was necessary to add special code to this model to ignore the top row and rightmost column of the grid, effectively converting an odd-sized grid into an even-sized one.  In the code, the "ppatches" variable holds all the patches except the excluded row and column, and the "ppatchat" procedure is like the built-in "patch-at" except for skipping that row and column.

See also the "Even Grid Example" in the Code Examples section of the Models Library.


CREDITS AND REFERENCES
-----------------------
Thanks to Ethan Bakshy for his help with this model.

For more information about lattice gas automata, see:

J. Hardy, Y. Pomeau & O. de Pazzis, Time evolution of two-dimensional model system. I. Invariant states and time correlation functions, J. Math. Phys. 14 (1973), pp. 1746-1759.

U. Frisch, B. Hasslacher & Y. Pomeau, Lattice-gas automata for the Navier-Stokes equation, Phys. Rev. Lett. 56 (1986), pp. 1505-1508. 

T. Toffoli and N. Margolus. 1987. "Cellular Automata Machines: A New Environment for Modeling". 

Wolfram, S. 2002. A New Kind of Science. Wolfram Media Inc.  Champaign, IL.

To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo Lattice Gas Automaton model. http://ccl.northwestern.edu/netlogo/models/LatticeGasAutomaton.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

circle
false
0
Circle -7566196 true true 35 35 230

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 46 130
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150
Polygon -7566196 true true 270 14
Polygon -7566196 true true 276 21

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

@#$#@#$#@
NetLogo 1.2pre8
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
