globals [
  bank-loans
  bank-reserves
  bank-deposits
  bank-to-loan
  clock
  xmax
  ymax
  rich 
  poor
  middle-class
  income-max
]
 
turtles-own [
  savings 
  loans
  wallet
  temp-loan
  wealth
  customer
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                     ;;;
;;;  Setup Procedures   ;;;
;;;                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;


to setup
  ca
  initialize-settings
  ask patches [set pcolor black]
  cct people [setup-turtles]
  poll-class
  setup-bank
  setup-plots
end


to initialize-settings
  set clock 2
  set rich 0
  set poor 0
  set middle-class 0
  set income-max 10
end

to setup-turtles  ;;Turtle Procedure
  set shape "person"
  rt random 360
  jump random (2 * max (list screen-edge-x screen-edge-y))  
  set wallet (random 2 * income-max)
  set savings 0
  set loans 0
  set wealth 0
  set customer -1
  get-color
end

to setup-bank
  set bank-loans 0
  set bank-reserves 0
  set bank-deposits 0
  set bank-to-loan 0
end

to setup-plots   
  set xmax 300
  set ymax (2 * money-total)
  clear-all-plots
  setup-plot1
  setup-plot2
  setup-plot3
  setup-histogram
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                     ;;;
;;; Run Time Procedures ;;;
;;;                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go 
  do-plots
  ask turtles [do-business]
  ask turtles [
    balance-books
    get-color
  ]
  bank-balance-sheet
  poll-class
end

;; polls the number in each corresponding economic class
to poll-class
  set rich (count turtles with [savings > income-max])
  set poor (count turtles with [loans > income-max])
  set middle-class (count turtles) - (rich + poor)
end

to do-business  ;;Turtle Procedure
  rt random 360
  fd 1  
  ;; turtle has money to trade with, and there is
  ;; another turtle to trade with on the same patch
  if ((savings > 0) or (wallet > 0) or (bank-to-loan > 0)) [
    set customer random-one-of other-turtles-here
    if customer != nobody and (random 2) = 0          ;;50% chance of trading
      [ifelse (random 2 = 0)                          ;;50% chance of trading $2 or $5, if trading
         [set wallet-of customer (wallet-of customer + 5)
          set wallet (wallet - 5)]
         [set wallet-of customer (wallet-of customer + 2)
          set wallet (wallet - 2)]
      ]
  ]
end

;; Check the balance of our wallet.
;; Put a positive balance in savings.  Try to get a loan to cover a 
;; negative balance.  If we cannot get a loan (if bank-to-loan < 0)
;; then maintain the negative wallet balance until the next round.
to balance-books  ;;Turtle Procedure
  ifelse (wallet < 0) 
   [ifelse (savings >= (0 - wallet))
      [withdraw-from-savings (0 - wallet)]
      [if (savings > 0)  
         [withdraw-from-savings savings]
       set temp-loan bank-to-loan
       ifelse (temp-loan >= (0 - wallet))
         [take-out-a-loan (0 - wallet)]
         [take-out-a-loan temp-loan]
      ]
   ]
   [deposit-to-savings wallet]

;; repay loans if savings are available  
  if (loans > 0) and (savings > 0) [
    ifelse (savings >= loans) 
      [withdraw-from-savings loans
       repay-a-loan loans]
      [withdraw-from-savings savings
       repay-a-loan wallet]
  ]
end


;; Sets aside required amount from liablilites into 
;; reserves, regardless of outstanding loans.  This may
;; result in a negative bank-to-loan amount, which
;; means that the bank will be unable to loan money
;; until it can set enough aside to account for reserves.
to bank-balance-sheet
  set bank-deposits sum (values-from turtles [savings])
  set bank-loans sum (values-from turtles [loans])
  set bank-reserves ((reserves / 100) * bank-deposits)
  set bank-to-loan (bank-deposits - (bank-reserves + bank-loans))
end


to deposit-to-savings [amount]  ;; Turtle Procedure
  set wallet (wallet - amount)
  set savings (savings + amount)
end 

to withdraw-from-savings [amount]  ;; Turtle Procedure
  set wallet (wallet + amount)
  set savings (savings - amount)
end


to repay-a-loan [amount]  ;; Turtle Procedure
  set loans (loans - amount)
  set wallet (wallet - amount)
  set bank-to-loan (bank-to-loan + amount)
end

to take-out-a-loan [amount]  ;; Turtle Procedure
  set loans (loans + amount)
  set wallet (wallet + amount)
  set bank-to-loan (bank-to-loan - amount)
end
  

;; color codes the rich (green), 
;; middle-class (gray), and poor (red)
to get-color ;;Turtle Procedure
  set color gray
  if (savings > income-max) [set color green]
  if (loans > income-max)  [set color red]
  set wealth (savings - loans)
end


to-report savings-total
  report sum (values-from turtles [savings])
end

to-report loans-total
  report sum (values-from turtles [loans])
end

to-report wallets-total
  report sum (values-from turtles [wallet])
end

to-report money-total
  report sum (values-from turtles [wallet + savings])
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                     ;;;
;;; Plotting Procedures ;;;
;;;                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

to do-plots
  set-current-plot "Money & Loans"
  set-current-plot-pen "money"
  plot money-total
  set-current-plot-pen "loans"
  plot loans-total

  set-current-plot "Savings & Wallets"
  set-current-plot-pen "savings"
  plot savings-total
  set-current-plot-pen "wallets"
  plot wallets-total

  set-current-plot "Income Dist"
  set-current-plot-pen "rich"
  plot rich
  set-current-plot-pen "middle"
  plot middle-class
  set-current-plot-pen "poor"
  plot poor

  every 1 [plot-histogram]
end


to plot-histogram
  locals
  [
    wealth-list
    min-wealth
    ave-wealth
    max-wealth
  ]
  set-current-plot "Wealth Dist Hist"
  set wealth-list values-from turtles [wealth]
  set min-wealth round (min wealth-list)
  set max-wealth round (max wealth-list)
  ifelse min-wealth < max-wealth
  [ set-plot-x-range min-wealth max-wealth ]
  [ set-plot-x-range min-wealth (min-wealth + 1) ]

  ;;draw grey line in center of distribution
  set-current-plot-pen "ave-wealth"
  plot-pen-reset
  set ave-wealth mean wealth-list
  plotxy ave-wealth 0
  plotxy ave-wealth people

  set-current-plot-pen "hist"
  set-histogram-num-bars 10
  histogram turtles [wealth]
end


to setup-plot1
  set-current-plot "Money & Loans"
  set-plot-x-range 0 xmax
  set-plot-y-range -50 ymax
end

to setup-plot2
  set-current-plot "Savings & Wallets"
  set-plot-x-range 0 xmax
  set-plot-y-range -50 ymax
end

to setup-plot3
  set-current-plot "Income Dist"
  set-plot-x-range 0 xmax
  set-plot-y-range 0 people
end


to setup-histogram
  set-current-plot "Wealth Dist Hist"
  set-plot-y-range 0 people
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Cash Flow model.
; http://ccl.northwestern.edu/netlogo/models/CashFlow.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
272
10
605
364
8
8
19.0
1
10
0
0

CC-WINDOW
610
41
892
158
Command Center

SLIDER
138
79
269
112
reserves
reserves
0.0
100.0
21.0
1.0
1
NIL

BUTTON
60
41
131
74
setup
setup
NIL
1
T
OBSERVER

SLIDER
4
79
131
112
people
people
0
500
473
1
1
NIL

PLOT
6
372
286
571
Money & Loans
Time
Mny + Lns
0.0
300.0
-50.0
1100.0
true
true
PENS
"money" 1.0 0 -16777216 true
"loans" 1.0 0 -65536 true

PLOT
307
372
588
571
Savings & Wallets
Time
Svngs + Wllts
0.0
300.0
-50.0
1100.0
true
true
PENS
"savings" 1.0 0 -11352576 true
"wallets" 1.0 0 -16776961 true

PLOT
609
372
894
571
Income Dist
Time
People
0.0
300.0
0.0
100.0
true
true
PENS
"rich" 1.0 0 -11352576 true
"middle" 1.0 0 -16777216 true
"poor" 1.0 0 -65536 true

PLOT
610
164
894
363
Wealth Dist Hist
Wealth
People
-50.0
50.0
0.0
100.0
false
false
PENS
"hist" 1.0 1 -16776961 true
"ave-wealth" 1.0 0 -7566196 true

MONITOR
38
118
131
167
Savings total
savings-total
2
1

MONITOR
138
117
240
166
Wallets total
Wallets-total
2
1

MONITOR
38
179
131
228
Loans total
loans-total
2
1

MONITOR
138
178
235
227
Money total
Money-total
2
1

BUTTON
138
41
209
74
go
go
T
1
T
OBSERVER

MONITOR
37
266
88
315
rich
rich
2
1

MONITOR
99
266
199
315
middle class
middle-class
2
1

MONITOR
210
266
260
315
poor
poor
2
1

TEXTBOX
93
239
209
257
Income Distribution:

@#$#@#$#@
WHAT IS IT?
-----------

This model is a simple extension of the model "Bank Reserves". The purpose of the model is to help the user examine whether there is a relationship between the reserve ratio that banks must keep and the degree of equality in the distribution of the money that exist in the system. 

It models the creation of money in an economy through a private banking system. As most of the money in the economy is kept in banks but only a little of it needs to be used (i.e. in cash form) at any one time, the banks need only keep a small portion of their savings on-hand for those transactions. This portion of the total savings is known as the banks' reserves. The banks are then able to loan out the rest of their savings. The government (the user in this case) sets a reserve ratio mandating how much of the banks' holdings must be kept in reserve at a given time. One 'super-bank' is used in this model to represent all banks in an economy. As this model demonstrates, the reserve ratio is the key determiner of how much money is created in the system. 

In each round, people (represented by turtles) interact with each other to simulate everyday economic activity. Given a randomly selected number, when a person is on the same patch as someone else it will either give the person two or five dollars, or no money at all. After this, people must then sort out the balance of their wallet with the bank. People will put a positive wallet balance in savings, or pay off a negative balance from funds already in savings. If the savings account is empty and the wallet has a negative balance, a person will take out a loan from the bank if funds are available to borrow (if bank-to-loan > 0). Otherwise, the person maintains the negative balance until the next round. Lastly, if someone has money in savings and money borrowed from the bank, that person will pay off as much of the loan as possible using the savings.

The number of rich people in each moment is calculated by enumerating the number of people whose savings exceed 10 dollars. The number of poor people is the number of people whose loans exceed 10 dollars. The rest of the people are considered to belong to the "middle-class".



HOW TO USE IT?
--------------

The RESERVES slider sets the banking reserve ratio (the percentage of money that a bank must keep in reserve at a given time). The PEOPLE slider sets the number of people that will be created in the model when the SETUP button is pressed. The SETUP button resets the model: it redistributes the patch colors, creates PEOPLE people and initializes all stored values. The GO button starts and stops the running of the model and the plotter.



1) THINGS TO NOTICE
-------------------
There are numerous display windows in the interface to help the user see where money in the economy is concentrated at a given time. SAVINGS-TOTAL indicates the total amount of money currently being kept in savings (and thus, in the banking system). LOANS-TOTAL is the amount the bank has lent out. WALLETS-TOTAL gives an indication of the total amount of money kept in the peoples' wallets. This figure may be negative at times when the bank has no more money to loan (the turtle will maintain a negative wallet balance until a loan is possible). MONEY-TOTAL indicates the total-amount of money currently in the economy (SAVINGS-TOTAL + WALLETS-TOTAL). Because WALLETS-TOTAL is generally kept at 0 in this model (we are assuming that everyone deposits all they can in savings), MONEY-TOTAL and SAVINGS TOTAL tend to be the same.

The three lower monitors show the number of rich, middle-class, or poor people. A person's color tells us whether he is rich (green color), middle-class (gray), or poor (red).

There are four plots to help you observe the operation of the economy. Plot 1 plots the number of LOANS-TOTAL and MONEY-TOTAL as a function of time.  Plot 2 plots the number of SAVINGS-TOTAL and WALLETS-TOTAL. Plot 3 plots income distribution and is the feature that you should pay the most attention to. The green pen plots the number of rich people; the black pen plots the number of middle-class people; the red pen plots the number of poor people. Finally, the fourth plot is a histogram that depicts how wealth is distributed among the population. Wealth is distributed in ten bars. Each bar corresponds to one tenth of the range of wealth.  The gray line in the histogram represents the mean wealth value of all the turtles.


2) THINGS TO TRY
----------------
Vary the RESERVES rate as the model runs, and watch the effect this has on income distribution. You can either watch the three Income Distribution monitors, or the Income Distribution plot.  Set RESERVES initially to 100 and observe the Income Distribution plot. Gradually lower RESERVES and see the effect on wealth distribution. Can you see whether the two of them are positively, negatively, or not related?



EXTENDING THE MODEL:
--------------------

Try extending the model to include payments of interest in the banking system. People with money deposited in savings should be credited with interest on their account (at a certain rate) from the bank from time to time. People with money on loan should make interest payments on their account to the bank from time to time.

This model has turtles interact in a very simple way to have money change hands (and create a need for loans). Try changing the model so that money moves around the system in a different way.



RELATED MODELS
--------------
See "Bank Reserves". 


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Cash Flow model. http://ccl.northwestern.edu/netlogo/models/CashFlow. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

person
false
0
Circle -7566196 true true 107 17 87
Polygon -7566196 true true 115 98 186 98 186 218 115 218
Polygon -7566196 true true 182 101 249 127 226 169 166 138
Polygon -7566196 true true 112 101 147 131 84 172 59 137
Polygon -7566196 true true 107 191 148 224 97 289 57 251
Polygon -7566196 true true 194 185 156 223 217 288 254 245

@#$#@#$#@
NetLogo 1.3beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
